package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.EvaluationFormDTO;
import digital.laboratory.platform.reagent.entity.EvaluationForm;
import digital.laboratory.platform.reagent.mapper.EvaluationFormMapper;
import digital.laboratory.platform.reagent.service.EvaluationFormService;
import digital.laboratory.platform.reagent.service.ProvideServicesOrSuppliesService;
import digital.laboratory.platform.reagent.service.PurchaseListDetailsService;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import digital.laboratory.platform.reagent.vo.EvaluationFormVO;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.List;

/**
 * (服务商/供应商评价表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (服务商 / 供应商评价表) 服务实现类
 */
@SuppressWarnings("all")
@Service
public class EvaluationFormServiceImpl extends ServiceImpl<EvaluationFormMapper, EvaluationForm> implements EvaluationFormService {

    @Autowired
    private EvaluationFormService evaluationFormService;
    @Autowired
    private ProvideServicesOrSuppliesService provideServicesOrSuppliesService;
    @Autowired
    private SupplierInformationService supplierInformationService;
    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Override//通过供应商ID查询评价表列表
    public List <EvaluationFormVO > getEvaluationFormVOList(String supplierInformationId) {

        List<EvaluationFormVO> evaluationFormVOList = baseMapper.getEvaluationFormVOList(supplierInformationId);

        return evaluationFormVOList;
    }

    @Override
    public EvaluationFormVO getFormById(String evaluationFormId) {

        EvaluationFormVO evaluationForm = baseMapper.getEvaluationForm(evaluationFormId);

        return evaluationForm;
    }

    @Override//增加评价表
    public EvaluationForm addFormById(String supplierInformationId) {

        EvaluationForm evaluationForm = new EvaluationForm();

        evaluationForm.setId(IdWorker.get32UUID().toUpperCase());

        evaluationForm.setSupplierInformationId(supplierInformationId);

        evaluationFormService.save(evaluationForm);

        return evaluationForm;
    }

    @Override//填写评价表并提交
    public EvaluationForm editFormById(EvaluationFormDTO evaluationFormDTO,DLPUser dlpUser) {

        EvaluationForm byId = evaluationFormService.getById(evaluationFormDTO.getEvaluationFormId());

        BeanUtils.copyProperties(evaluationFormDTO, byId);

        byId.setPrimaryUserId(dlpUser.getId());

        byId.setCommentsDateFromPrimary(LocalDate.now());

        byId.setStatus(1);

        if (evaluationFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("提交失败"));
    }

    @Override//二级审核
    public EvaluationForm auditFormOfSecondary(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        EvaluationForm byId = evaluationFormService.getById(auditAndApproveDTO.getUuId());

        byId.setSecondaryUserId(dlpUser.getId());

        byId.setCommentsDateFromSecondary(LocalDate.now());

        byId.setCommentsFromSecondary(auditAndApproveDTO.getAuditOpinion());

        byId.setCommentsResultFromSecondary(auditAndApproveDTO.getAuditResult());

        if (evaluationFormService.updateById(byId)) {
            return byId;
        } else {
            return null;
        }
    }

    @Override//三级审核

    public EvaluationForm auditFormOfThreeLevel(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        EvaluationForm byId = evaluationFormService.getById(auditAndApproveDTO.getUuId());

        byId.setSecondaryUserId(dlpUser.getId());

        byId.setCommentsDateFromThreeLevel(LocalDate.now());

        byId.setCommentsFromThreeLevel(auditAndApproveDTO.getAuditOpinion());

        byId.setCommentsResultFromThreeLevel(auditAndApproveDTO.getAuditResult());

        byId.setStatus(2);

        if (evaluationFormService.updateById(byId)) {
            return byId;
        } else {
            return null;
        }
    }


}









