package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.reagent.config.PageUtils;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.ReagentConsumableInventoryMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.BatchDetailsVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumableInventoryFullVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumableInventoryVO;
import io.swagger.models.auth.In;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 试剂耗材库存服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 试剂耗材库存 服务实现类
 */
@Service
public class ReagentConsumableInventoryServiceImpl extends ServiceImpl<ReagentConsumableInventoryMapper, ReagentConsumableInventory> implements ReagentConsumableInventoryService {

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;
    @Autowired
    private BatchDetailsService batchDetailsService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Override//试剂耗材管理列表
    public Page<ReagentConsumableInventoryVO> getReagentConsumableInventoryVOList(Page page, String name) {

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

        if (name != null) {

            reagentConsumableInventoryQueryWrapper.eq("category", "试剂").or()
                    .eq("category", "耗材")
                    .like("reagentConsumableInventory_name",name);
        } else {

            reagentConsumableInventoryQueryWrapper.eq("category", "试剂").or()
                    .eq("category", "耗材");
        }

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(reagentConsumableInventoryQueryWrapper);

        ArrayList<ReagentConsumableInventoryVO> reagentConsumableInventoryVOS = new ArrayList<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventory.getReagentConsumableInventoryId());

            ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();

            BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryVO);
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);

            ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableInventory.getReagentConsumableId());
            //赋值试剂耗材名称
            reagentConsumableInventoryVO.setReagentConsumablesName(byId.getReagentConsumableName());

            reagentConsumableInventoryVOS.add(reagentConsumableInventoryVO);
        }
        if (reagentConsumableInventoryVOS != null) {
            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOS);

            return pages;
        } else return null;

    }

    @Override//标准物质管理列表
    public Page<ReagentConsumableInventoryVO> getReagentConsumableInventoryRMVOList(Page page, String name) {

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

        if (name != null) {
            reagentConsumableInventoryQueryWrapper.eq("category", "标准物质").or()
                    .eq("category", "标准储备溶液")
                    .like("reagentConsumable_name", name);
        } else {
            reagentConsumableInventoryQueryWrapper.eq("category", "标准物质").or()
                    .eq("category", "标准储备溶液");
        }

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(reagentConsumableInventoryQueryWrapper);

        ArrayList<ReagentConsumableInventoryVO> reagentConsumableInventoryVOS = new ArrayList<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventory.getReagentConsumableInventoryId());

            ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();

            BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryVO);
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);

            ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableInventory.getReagentConsumableId());
            //赋值试剂耗材名称
            reagentConsumableInventoryVO.setReagentConsumablesName(byId.getReagentConsumableName());

            reagentConsumableInventoryVOS.add(reagentConsumableInventoryVO);
        }
        if (reagentConsumableInventoryVOS != null) {
            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOS);

            return pages;
        } else return null;
    }


    @Override//领用减少库存
    public ReagentConsumableInventory addById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() + quantity);

        if (reagentConsumableInventoryService.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存添加失败"));
    }

    @Override//归还/入库增加库存
    public ReagentConsumableInventory reduceById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() - quantity);

        if (reagentConsumableInventoryService.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存减少失败"));
    }


    @Override//标准物质/溶液停用
    public ReferenceMaterial deactivateById(String referenceMaterialId) {

        ReferenceMaterial byId = referenceMaterialService.getById(referenceMaterialId);

        byId.setStatus(-1);

        return byId;
    }

    @Override//分页查询标准物质
    public Page<ReagentConsumableInventoryFullVO> getAllList(Integer current, Integer size, String reagentConsumableName) {

        if (reagentConsumableName != null) {

            QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

            reagentConsumableInventoryQueryWrapper.like("reagent_consumable_name", reagentConsumableName);

            List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(reagentConsumableInventoryQueryWrapper);

            List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

            for (ReagentConsumableInventory reagentConsumableInventory : list) {

                LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();

                referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId());

                List<ReferenceMaterial> list1 = referenceMaterialService.list(referenceMaterialLambdaQueryWrapper);

                for (ReferenceMaterial referenceMaterial : list1) {

                    ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                    BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                    reagentConsumableInventoryFullVO.setNumber(referenceMaterial.getNumber());

                    reagentConsumableInventoryFullVO.setBatchDetailsId(referenceMaterial.getBatchDetailsId());

                    BatchDetails batchDetails = batchDetailsService.getById(referenceMaterial.getBatchDetailsId());

                    SupplierInformation supplierInformation = supplierInformationService.getById(batchDetails.getSupplierId());

                    reagentConsumableInventoryFullVO.setSupplierName(supplierInformation.getSupplierName());

                    reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                    reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);

                }
            }
            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages(current, size, reagentConsumableInventoryFullVOList);

            return pages;
        }


        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

        List<ReferenceMaterial> referenceMaterialList = referenceMaterialService.list();

        for (ReferenceMaterial referenceMaterial : referenceMaterialList) {

            if (referenceMaterial.getStatus() == 0) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                ReagentConsumableInventory byId = reagentConsumableInventoryService.getById(referenceMaterial.getReagentConsumableInventoryId());

                BeanUtils.copyProperties(byId, reagentConsumableInventoryFullVO);

                reagentConsumableInventoryFullVO.setNumber(referenceMaterial.getNumber());

                reagentConsumableInventoryFullVO.setBatchDetailsId(referenceMaterial.getBatchDetailsId());

                BatchDetails batchDetails = batchDetailsService.getById(referenceMaterial.getBatchDetailsId());

                SupplierInformation supplierInformation = supplierInformationService.getById(batchDetails.getSupplierId());

                reagentConsumableInventoryFullVO.setSupplierName(supplierInformation.getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
            }
        }

        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages(current, size, reagentConsumableInventoryFullVOList);

        return pages;

    }

    //分页查询试剂耗材
    @Override
    public Page<ReagentConsumableInventoryFullVO> getAllRM(Integer current, Integer size, String reagentConsumableName) {

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

        if (reagentConsumableName != null) {

            QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

            reagentConsumableInventoryQueryWrapper.like("reagent_consumable_name", reagentConsumableName);

            List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(reagentConsumableInventoryQueryWrapper);

            for (ReagentConsumableInventory reagentConsumableInventory : list) {

                LambdaQueryWrapper<BatchDetails> batchDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

                batchDetailsLambdaQueryWrapper.eq(BatchDetails::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId());

                List<BatchDetails> list1 = batchDetailsService.list(batchDetailsLambdaQueryWrapper);

                for (BatchDetails batchDetails : list1) {

                    ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                    BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                    SupplierInformation byId = supplierInformationService.getById(batchDetails.getSupplierId());

                    reagentConsumableInventoryFullVO.setSupplierName(byId.getSupplierName());

                    reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                    reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                    reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
                }

            }

            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages(current, size, reagentConsumableInventoryFullVOList);

            return pages;
        }

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

        reagentConsumableInventoryQueryWrapper.eq("category", "试剂")
                .or().eq("category", "耗材");

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(reagentConsumableInventoryQueryWrapper);

        LambdaQueryWrapper<BatchDetails> batchDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            if (reagentConsumableInventory.getTotalQuantity() > 0) {

                batchDetailsLambdaQueryWrapper.eq(BatchDetails::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId());

                List<BatchDetails> batchDetailsList = batchDetailsService.list(batchDetailsLambdaQueryWrapper);

                for (BatchDetails batchDetails : batchDetailsList) {

                    ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                    BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                    SupplierInformation byId = supplierInformationService.getById(batchDetails.getSupplierId());

                    reagentConsumableInventoryFullVO.setSupplierName(byId.getSupplierName());

                    reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                    reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);

                    reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());
                }

            }
        }
        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages(current, size, reagentConsumableInventoryFullVOList);

        return pages;

    }


}
